@@ -1,5 +1,5 @@
 /* comm -- compare two sorted files line by line.
-   Copyright (C) 86, 90, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 86, 90, 91, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -159,7 +159,7 @@ compare_files (char **infiles)
 	  return 1;
 	}
 
-      thisline[i] = readline (thisline[i], streams[i]);
+      thisline[i] = readlinebuffer (thisline[i], streams[i]);
     }
 
   while (thisline[0] || thisline[1])
@@ -199,9 +199,9 @@ compare_files (char **infiles)
       /* Step the file the line came from.
 	 If the files match, step both files.  */
       if (order >= 0)
-	thisline[1] = readline (thisline[1], streams[1]);
+	thisline[1] = readlinebuffer (thisline[1], streams[1]);
       if (order <= 0)
-	thisline[0] = readline (thisline[0], streams[0]);
+	thisline[0] = readlinebuffer (thisline[0], streams[0]);
     }
 
   /* Free all storage and close all input streams. */
@@ -222,6 +222,7 @@ main (int argc, char **argv)
 {
   int c;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -261,7 +262,10 @@ main (int argc, char **argv)
       }
 
   if (optind + 2 != argc)
-    usage (EXIT_FAILURE);
+    {
+      error (0, 0, _("too few arguments"));
+      usage (EXIT_FAILURE);
+    }
 
   exit (compare_files (argv + optind) == 0
 	? EXIT_SUCCESS : EXIT_FAILURE);
