@@ -1,5 +1,5 @@
 /* copy.c -- core functions for copying files and directories
-   Copyright (C) 89, 90, 91, 1995-2002 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2003 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,10 +17,6 @@
 
 /* Extracted from cp.c and librarified by Jim Meyering.  */
 
-#ifdef _AIX
- #pragma alloca
-#endif
-
 #include <config.h>
 #include <stdio.h>
 #include <assert.h>
@@ -72,7 +68,7 @@ struct dir_list
 /* Describe a just-created or just-renamed destination file.  */
 struct F_triple
 {
-  char const* name;
+  char *name;
   ino_t st_ino;
   dev_t st_dev;
 };
@@ -424,12 +420,22 @@ close_src_desc:
    making the `copy' operation remove both copies of the file
    in that case, while still allowing the user to e.g., move or
    copy a regular file onto a symlink that points to it.
-   Try to minimize the cost of this function in the common case.  */
+   Try to minimize the cost of this function in the common case.
+   Set *RETURN_NOW if we've determined that the caller has no more
+   work to do and should return successfully, right away.
+
+   Set *UNLINK_SRC if we've determined that the caller wants to do
+   `rename (a, b)' where `a' and `b' are hard links to the same file.
+   In that case, the caller should try to unlink `a' and then return
+   successfully.  Ideally, we wouldn't have to do that, and we'd be
+   able to rely on rename to remove the source file.  However, POSIX
+   mistakenly requires that such a rename call do *nothing* and return
+   successfully.  */
 
 static int
 same_file_ok (const char *src_path, const struct stat *src_sb,
 	      const char *dst_path, const struct stat *dst_sb,
-	      const struct cp_options *x, int *return_now)
+	      const struct cp_options *x, int *return_now, int *unlink_src)
 {
   const struct stat *src_sb_link;
   const struct stat *dst_sb_link;
@@ -440,6 +446,7 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
   int same = (SAME_INODE (*src_sb, *dst_sb));
 
   *return_now = 0;
+  *unlink_src = 0;
 
   /* FIXME: this should (at the very least) be moved into the following
      if-block.  More likely, it should be removed, because it inhibits
@@ -548,10 +555,21 @@ same_file_ok (const char *src_path, const struct stat *src_sb,
      destination file before opening it -- via `rename' if they're on
      the same file system, via `unlink (DST_PATH)' otherwise.
      It's also ok if they're distinct hard links to the same file.  */
-  if ((x->move_mode || x->unlink_dest_before_opening)
-      && (S_ISLNK (dst_sb_link->st_mode)
-	  || (same_link && !same_name (src_path, dst_path))))
-    return 1;
+  if (x->move_mode || x->unlink_dest_before_opening)
+    {
+      if (S_ISLNK (dst_sb_link->st_mode))
+	return 1;
+
+      if (same_link && !same_name (src_path, dst_path))
+	{
+	  if (x->move_mode)
+	    {
+	      *unlink_src = 1;
+	      *return_now = 1;
+	    }
+	  return 1;
+	}
+    }
 
   /* If neither is a symlink, then it's ok as long as they aren't
      hard links to the same file.  */
@@ -662,7 +680,7 @@ static void
 triple_free (void *x)
 {
   struct F_triple *a = x;
-  free ((char *) (a->name));
+  free (a->name);
   free (a);
 }
 
@@ -713,7 +731,7 @@ seen_file (Hash_table const *ht, char const *filename,
   if (ht == NULL)
     return 0;
 
-  new_ent.name = filename;
+  new_ent.name = (char *) filename;
   new_ent.st_ino = stats->st_ino;
   new_ent.st_dev = stats->st_dev;
 
@@ -734,7 +752,7 @@ record_file (Hash_table *ht, char const *filename,
   if (ht == NULL)
     return;
 
-  ent = (struct F_triple *) xmalloc (sizeof *ent);
+  ent = xmalloc (sizeof *ent);
   ent->name = xstrdup (filename);
   if (stats)
     {
@@ -856,8 +874,21 @@ copy_internal (const char *src_path, const char *dst_path,
       else
 	{
 	  int return_now;
+	  int unlink_src;
 	  int ok = same_file_ok (src_path, &src_sb, dst_path, &dst_sb,
-				 x, &return_now);
+				 x, &return_now, &unlink_src);
+	  if (unlink_src)
+	    {
+	      if (unlink (src_path))
+		{
+		  error (0, errno, _("cannot remove %s"), quote (src_path));
+		  return 1;
+		}
+	      /* Tell the caller that there's no need to remove src_path.  */
+	      if (rename_succeeded)
+		*rename_succeeded = 1;
+	    }
+
 	  if (return_now)
 	    return 0;
 
@@ -1110,14 +1141,32 @@ copy_internal (const char *src_path, const char *dst_path,
 	  goto un_backup;
 	}
 
-      if (link (earlier_file, dst_path))
-	{
-	  error (0, errno, _("cannot create hard link %s to %s"),
-		 quote_n (0, dst_path), quote_n (1, earlier_file));
-	  goto un_backup;
-	}
+      {
+	int link_failed;
 
-      return 0;
+	link_failed = link (earlier_file, dst_path);
+
+	/* If the link failed because of an existing destination,
+	   remove that file and then call link again.  */
+	if (link_failed && errno == EEXIST)
+	  {
+	    if (unlink (dst_path))
+	      {
+		error (0, errno, _("cannot remove %s"), quote (dst_path));
+		goto un_backup;
+	      }
+	    link_failed = link (earlier_file, dst_path);
+	  }
+
+	if (link_failed)
+	  {
+	    error (0, errno, _("cannot create hard link %s to %s"),
+		   quote_n (0, dst_path), quote_n (1, earlier_file));
+	    goto un_backup;
+	  }
+
+	return 0;
+      }
     }
 
   if (x->move_mode)
@@ -1509,10 +1558,16 @@ copy_internal (const char *src_path, const char *dst_path,
   /* Preserve the st_author field.  */
   {
     file_t file = file_name_lookup (dst_path, 0, 0);
-    if (file_chauthor (file, src_sb.st_author))
-      error (0, errno, _("failed to preserve authorship for %s"),
-	     quote (dst_path));
-    mach_port_deallocate (mach_task_self (), file);
+    if (file == MACH_PORT_NULL)
+      error (0, errno, _("failed to lookup file %s"), quote (dst_path));
+    else
+      {
+	error_t err = file_chauthor (file, src_sb.st_author);
+	if (err)
+	  error (0, err, _("failed to preserve authorship for %s"),
+		 quote (dst_path));
+	mach_port_deallocate (mach_task_self (), file);
+      }
   }
 #endif
 
@@ -1538,11 +1593,14 @@ copy_internal (const char *src_path, const char *dst_path,
 
 un_backup:
 
-  /* We didn't create the destination.
-     Remove the entry associating the source dev/ino with the
+  /* We have failed to create the destination file.
+     If we've just added a dev/ino entry via the remember_copied
+     call above (i.e., unless we've just failed to create a hard link),
+     remove the entry associating the source dev/ino with the
      destination file name, so we don't try to `preserve' a link
      to a file we didn't create.  */
-  forget_created (src_sb.st_ino, src_sb.st_dev);
+  if (earlier_file == NULL)
+    forget_created (src_sb.st_ino, src_sb.st_dev);
 
   if (dst_backup)
     {
