@@ -17,10 +17,6 @@
 
    Written by Torbjorn Granlund, David MacKenzie, and Jim Meyering. */
 
-#ifdef _AIX
- #pragma alloca
-#endif
-
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -414,7 +410,7 @@ make_path_private (const char *const_dirpath, int src_offset, int mode,
 	  /* Add this directory to the list of directories whose modes need
 	     fixing later. */
 	  struct dir_attr *new =
-	    (struct dir_attr *) xmalloc (sizeof (struct dir_attr));
+	    xmalloc (sizeof (struct dir_attr));
 	  new->slash_offset = slash - dirpath;
 	  new->next = *attr_list;
 	  *attr_list = new;
@@ -541,18 +537,18 @@ do_copy (int n_files, char **file, const char *target_directory,
 
   if (!dest_is_dir)
     {
-      if (target_directory)
+      if (target_directory || 1 < n_files)
 	{
-	  error (0, 0, _("%s: specified target is not a directory"),
-		 quote (dest));
-	  usage (EXIT_FAILURE);
-	}
+	  char const *msg;
+	  if (new_dst)
+	    msg = N_("%s: specified destination directory does not exist");
+	  else if (target_directory)
+	    msg = N_("%s: specified target is not a directory");
+	  else /* n_files > 1 */
+	    msg =
+	  N_("copying multiple files, but last argument %s is not a directory");
 
-      if (n_files > 1)
-	{
-	  error (0, 0,
-	 _("copying multiple files, but last argument %s is not a directory"),
-	     quote (dest));
+	  error (0, 0, _(msg), quote (dest));
 	  usage (EXIT_FAILURE);
 	}
     }
@@ -606,8 +602,8 @@ do_copy (int n_files, char **file, const char *target_directory,
 						  &attr_list, &new_dst,
 						  x->xstat);
 	    }
-  	  else
-  	    {
+	  else
+	    {
 	      char *arg_base;
 	      /* Append the last component of `arg' to `dest'.  */
 
@@ -622,7 +618,7 @@ do_copy (int n_files, char **file, const char *target_directory,
 	    {
 	      /* make_path_private failed, so don't even attempt the copy. */
 	      ret = 1;
-  	    }
+	    }
 	  else
 	    {
 	      int copy_into_self;
@@ -837,6 +833,7 @@ main (int argc, char **argv)
   int copy_contents = 0;
   char *target_directory = NULL;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
