@@ -1,5 +1,5 @@
 /* install - copy files and set attributes
-   Copyright (C) 89, 90, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,10 +17,6 @@
 
 /* Written by David MacKenzie <djm@gnu.ai.mit.edu> */
 
-#ifdef _AIX
- #pragma alloca
-#endif
-
 #include <config.h>
 #include <stdio.h>
 #include <getopt.h>
@@ -49,10 +45,6 @@
 # include <sys/wait.h>
 #endif
 
-#if HAVE_VALUES_H
-# include <values.h>
-#endif
-
 struct passwd *getpwnam ();
 struct group *getgrnam ();
 
@@ -82,19 +74,19 @@ int isdir ();
 
 int stat ();
 
-static int change_timestamps PARAMS ((const char *from, const char *to));
-static int change_attributes PARAMS ((const char *path));
-static int copy_file PARAMS ((const char *from, const char *to,
-			      const struct cp_options *x));
-static int install_file_to_path PARAMS ((const char *from, const char *to,
-					 const struct cp_options *x));
-static int install_file_in_dir PARAMS ((const char *from, const char *to_dir,
-					const struct cp_options *x));
-static int install_file_in_file PARAMS ((const char *from, const char *to,
-					 const struct cp_options *x));
-static void get_ids PARAMS ((void));
-static void strip PARAMS ((const char *path));
-void usage PARAMS ((int status));
+static int change_timestamps (const char *from, const char *to);
+static int change_attributes (const char *path);
+static int copy_file (const char *from, const char *to,
+		      const struct cp_options *x);
+static int install_file_to_path (const char *from, const char *to,
+				 const struct cp_options *x);
+static int install_file_in_dir (const char *from, const char *to_dir,
+				const struct cp_options *x);
+static int install_file_in_file (const char *from, const char *to,
+				 const struct cp_options *x);
+static void get_ids (void);
+static void strip (const char *path);
+void usage (int status);
 
 /* The name this program was run with, for error messages. */
 char *program_name;
@@ -191,6 +183,7 @@ main (int argc, char **argv)
   int n_files;
   char **file;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -283,7 +276,7 @@ main (int argc, char **argv)
   n_files = argc - optind;
   file = argv + optind;
 
-  if (n_files == 0 || (n_files == 1 && !dir_arg))
+  if (argc <= optind || (n_files == 1 && !dir_arg))
     {
       error (0, 0, _("too few arguments"));
       usage (EXIT_FAILURE);
@@ -472,13 +465,13 @@ change_attributes (const char *path)
 #endif
       )
     {
-      error (0, errno, "cannot change ownership of %s", quote (path));
+      error (0, errno, _("cannot change ownership of %s"), quote (path));
       err = 1;
     }
 
   if (!err && chmod (path, mode))
     {
-      error (0, errno, "cannot change permissions of %s", quote (path));
+      error (0, errno, _("cannot change permissions of %s"), quote (path));
       err = 1;
     }
 
@@ -558,11 +551,11 @@ get_ids (void)
       pw = getpwnam (owner_name);
       if (pw == NULL)
 	{
-	  long int tmp_long;
-	  if (xstrtol (owner_name, NULL, 0, &tmp_long, NULL) != LONGINT_OK
-	      || tmp_long < 0 || tmp_long > UID_T_MAX)
+	  unsigned long int tmp;
+	  if (xstrtoul (owner_name, NULL, 0, &tmp, NULL) != LONGINT_OK
+	      || UID_T_MAX < tmp)
 	    error (EXIT_FAILURE, 0, _("invalid user %s"), quote (owner_name));
-	  owner_id = (uid_t) tmp_long;
+	  owner_id = tmp;
 	}
       else
 	owner_id = pw->pw_uid;
@@ -576,11 +569,11 @@ get_ids (void)
       gr = getgrnam (group_name);
       if (gr == NULL)
 	{
-	  long int tmp_long;
-	  if (xstrtol (group_name, NULL, 0, &tmp_long, NULL) != LONGINT_OK
-	      || tmp_long < 0 || tmp_long > GID_T_MAX)
+	  unsigned long int tmp;
+	  if (xstrtoul (group_name, NULL, 0, &tmp, NULL) != LONGINT_OK
+	      || GID_T_MAX < tmp)
 	    error (EXIT_FAILURE, 0, _("invalid group %s"), quote (group_name));
-	  group_id = (gid_t) tmp_long;
+	  group_id = tmp;
 	}
       else
 	group_id = gr->gr_gid;
