@@ -1,5 +1,5 @@
 /* od -- dump files in octal and other formats
-   Copyright (C) 92, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 92, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ struct tspec
   {
     enum output_format fmt;
     enum size_spec size;
-    void (*print_function) PARAMS ((size_t, const char *, const char *));
+    void (*print_function) (size_t, const char *, const char *);
     char *fmt_string;
     int hexl_mode_trailer;
     int field_width;
@@ -193,7 +193,7 @@ static uintmax_t pseudo_offset;
 
 /* Function that accepts an address and an optional following char,
    and prints the address and char to stdout.  */
-static void (*format_address) PARAMS ((uintmax_t, char));
+static void (*format_address) (uintmax_t, char);
 
 /* The number of input bytes to skip before formatting and writing.  */
 static uintmax_t n_bytes_to_skip = 0;
@@ -676,7 +676,7 @@ decode_one_format (const char *s_orig, const char *s, const char **next,
   enum output_format fmt;
   const char *pre_fmt_string;
   char *fmt_string;
-  void (*print_function) PARAMS ((size_t, const char *, const char *));
+  void (*print_function) (size_t, const char *, const char *);
   const char *p;
   unsigned int c;
   unsigned int field_width = 0;
@@ -1041,9 +1041,7 @@ decode_format_string (const char *s)
       if (n_specs >= n_specs_allocated)
 	{
 	  n_specs_allocated = 1 + (3 * n_specs_allocated) / 2;
-	  spec = (struct tspec *) xrealloc ((char *) spec,
-					    (n_specs_allocated
-					     * sizeof (struct tspec)));
+	  spec = xrealloc (spec, (n_specs_allocated * sizeof (struct tspec)));
 	}
 
       memcpy ((char *) &spec[n_specs], (char *) &tspec,
@@ -1628,6 +1626,7 @@ main (int argc, char **argv)
      after any true address.  */
   uintmax_t pseudo_start IF_LINT (= 0);
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -1662,7 +1661,7 @@ main (int argc, char **argv)
 
   n_specs = 0;
   n_specs_allocated = 5;
-  spec = (struct tspec *) xmalloc (n_specs_allocated * sizeof (struct tspec));
+  spec = xmalloc (n_specs_allocated * sizeof (struct tspec));
 
   format_address = format_address_std;
   address_base = 8;
@@ -1920,7 +1919,7 @@ it must be one character from [doxn]"),
     {
       end_offset = n_bytes_to_skip + max_bytes_to_format;
       if (end_offset < n_bytes_to_skip)
-	error (EXIT_FAILURE, 0, "skip-bytes + read-bytes is too large");
+	error (EXIT_FAILURE, 0, _("skip-bytes + read-bytes is too large"));
     }
 
   if (n_specs == 0)
