@@ -1,5 +1,5 @@
 /* pathchk -- check whether pathnames are valid or portable
-   Copyright (C) 1991-2002 Free Software Foundation, Inc.
+   Copyright (C) 1991-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -66,6 +66,7 @@ extern int errno;
 # endif /* not PATH_MAX */
 # ifndef NAME_MAX
 #  define NAME_MAX_FOR(p) pathconf_wrapper ((p), _PC_NAME_MAX);
+#  undef NEED_PATHCONF_WRAPPER
 #  define NEED_PATHCONF_WRAPPER 1
 # endif /* not NAME_MAX */
 
@@ -104,7 +105,7 @@ extern int errno;
 # define NAME_MAX_FOR(p) NAME_MAX
 #endif
 
-static int validate_path PARAMS ((char *path, int portability));
+static int validate_path (char *path, int portability);
 
 /* The name this program was run with. */
 char *program_name;
@@ -162,6 +163,7 @@ main (int argc, char **argv)
   int check_portability = 0;
   int optc;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -231,7 +233,7 @@ portable_chars_only (const char *path)
   const char *p;
 
   for (p = path; *p; ++p)
-    if (portable_chars[(const unsigned char) *p] == 0)
+    if (portable_chars[(unsigned char) *p] == 0)
       {
 	error (0, 0, _("path `%s' contains nonportable character `%c'"),
 	       path, *p);
