@@ -17,10 +17,6 @@
 
 /* Extracted from rm.c and librarified, then rewritten by Jim Meyering.  */
 
-#ifdef _AIX
- #pragma alloca
-#endif
-
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -55,6 +51,10 @@
 # define ROOT_CAN_UNLINK_DIRS 1
 #endif
 
+#ifndef HAVE_WORKING_READDIR
+# define HAVE_WORKING_READDIR 0
+#endif
+
 enum Ternary
   {
     T_UNKNOWN = 2,
@@ -235,7 +235,7 @@ pop_dir (Dirstack_state *ds)
 
   /* Pop the length stack, too.  */
   assert (obstack_object_size (&ds->len_stack) >= sizeof (size_t));
-  obstack_blank (&ds->len_stack, (int) -(sizeof (size_t)));
+  obstack_blank (&ds->len_stack, -(int) sizeof (size_t));
 }
 
 /* Copy the SRC_LEN bytes of data beginning at SRC into the DST_LEN-byte
@@ -359,7 +359,7 @@ AD_stack_pop (Dirstack_state *ds)
   struct AD_ent *top = AD_stack_top (ds);
   if (top->unremovable)
     hash_free (top->unremovable);
-  obstack_blank (&ds->Active_dir, -sizeof (struct AD_ent));
+  obstack_blank (&ds->Active_dir, -(int) sizeof (struct AD_ent));
   pop_dir (ds);
 }
 
@@ -397,8 +397,9 @@ AD_pop_and_chdir (Dirstack_state *ds)
     }
   else
     {
-      if (restore_cwd (&top->u.saved_cwd, NULL, NULL))
-	exit (EXIT_FAILURE);
+      if (restore_cwd (&top->u.saved_cwd))
+	error (EXIT_FAILURE, errno,
+	       _("failed to return to initial working directory"));
     }
 
   if (lstat (".", &sb))
@@ -507,36 +508,37 @@ AD_is_removable (Dirstack_state const *ds, char const *file)
   return ! (top->unremovable && hash_lookup (top->unremovable, file));
 }
 
+/* A wrapper for readdir so that callers don't see entries for `.' or `..'.  */
+static struct dirent *
+readdir_ignoring_dotdirs (DIR *dirp)
+{
+  while (1)
+    {
+      struct dirent *dp = readdir (dirp);
+      if (dp == NULL || ! DOT_OR_DOTDOT (dp->d_name))
+	return dp;
+    }
+}
+
+/* Return nonzero if DIR is determined to be an empty directory
+   or if opendir or readdir fails.  */
 static bool
 is_empty_dir (char const *dir)
 {
   DIR *dirp = opendir (dir);
+  struct dirent *dp;
+  int saved_errno;
+
   if (dirp == NULL)
-    {
-      closedir (dirp);
-      return false;
-    }
+    return false;
 
-  while (1)
-    {
-      struct dirent *dp;
-      const char *f;
-
-      errno = 0;
-      dp = readdir (dirp);
-      if (dp == NULL)
-	{
-	  closedir (dirp);
-	  return errno == 0 ? true : false;
-	}
-
-      f = dp->d_name;
-      if ( ! DOT_OR_DOTDOT (f))
-	{
-	  closedir (dirp);
-	  return false;
-	}
-    }
+  errno = 0;
+  dp = readdir_ignoring_dotdirs (dirp);
+  saved_errno = errno;
+  closedir (dirp);
+  if (dp != NULL)
+    return false;
+  return saved_errno == 0 ? true : false;
 }
 
 /* Prompt whether to remove FILENAME, if required via a combination of
@@ -742,7 +744,7 @@ remove_entry (Dirstack_state const *ds, char const *filename,
     }
 
 
-#else
+#else /* ! ROOT_CAN_UNLINK_DIRS */
 
   if (is_dir == T_YES && ! x->recursive)
     {
@@ -802,6 +804,7 @@ remove_cwd_entries (Dirstack_state *ds, char **subdir, struct stat *subdir_sb,
   DIR *dirp = opendir (".");
   struct AD_ent *top = AD_stack_top (ds);
   enum RM_status status = top->status;
+  bool need_rewinddir = false;
 
   assert (VALID_STATUS (status));
   *subdir = NULL;
@@ -825,7 +828,7 @@ remove_cwd_entries (Dirstack_state *ds, char **subdir, struct stat *subdir_sb,
       /* Set errno to zero so we can distinguish between a readdir failure
 	 and when readdir simply finds that there are no more entries.  */
       errno = 0;
-      if ((dp = readdir (dirp)) == NULL)
+      if ((dp = readdir_ignoring_dotdirs (dirp)) == NULL)
 	{
 	  if (errno)
 	    {
@@ -837,12 +840,18 @@ remove_cwd_entries (Dirstack_state *ds, char **subdir, struct stat *subdir_sb,
 	      /* Arrange to give a diagnostic after exiting this loop.  */
 	      dirp = NULL;
 	    }
+	  else if (need_rewinddir)
+	    {
+	      /* On buggy systems, call rewinddir if we've called unlink
+		 or rmdir since the opendir or a previous rewinddir.  */
+	      rewinddir (dirp);
+	      need_rewinddir = false;
+	      continue;
+	    }
 	  break;
 	}
 
       f = dp->d_name;
-      if (DOT_OR_DOTDOT (f))
-	continue;
 
       /* Skip files we've already tried/failed to remove.  */
       if ( ! AD_is_removable (ds, f))
@@ -856,7 +865,9 @@ remove_cwd_entries (Dirstack_state *ds, char **subdir, struct stat *subdir_sb,
       switch (tmp_status)
 	{
 	case RM_OK:
-	  /* do nothing */
+	  /* On buggy systems, record the fact that we've just
+	     removed a directory entry.  */
+	  need_rewinddir = ! HAVE_WORKING_READDIR;
 	  break;
 
 	case RM_ERROR:
@@ -866,34 +877,51 @@ remove_cwd_entries (Dirstack_state *ds, char **subdir, struct stat *subdir_sb,
 	  break;
 
 	case RM_NONEMPTY_DIR:
-	  /* Record dev/ino of F so that we can compare
-	     that with dev/ino of `.' after the chdir.
-	     This dev/ino pair is also used in cycle detection.  */
-	  if (lstat (f, subdir_sb))
-	    error (EXIT_FAILURE, errno, _("cannot lstat %s"),
-		   quote (full_filename (f)));
+	  {
+	    /* Save a copy of errno, in case the preceding unlink (from
+	       remove_entry's DO_UNLINK) of a non-directory failed due
+	       to EPERM.  */
+	    int saved_errno = errno;
 
-	  if (chdir (f))
-	    {
-	      error (0, errno, _("cannot chdir from %s to %s"),
-		     quote_n (0, full_filename (".")), quote_n (1, f));
-	      AD_mark_as_unremovable (ds, f);
-	      status = RM_ERROR;
-	      break;
-	    }
-	  if (cycle_check (&ds->cycle_check_state, subdir_sb))
-	    {
-	      error (0, 0, _("\
+	    /* Record dev/ino of F so that we can compare
+	       that with dev/ino of `.' after the chdir.
+	       This dev/ino pair is also used in cycle detection.  */
+	    if (lstat (f, subdir_sb))
+	      error (EXIT_FAILURE, errno, _("cannot lstat %s"),
+		     quote (full_filename (f)));
+
+	    if (chdir (f))
+	      {
+		/* It is much more common that we reach this point for an
+		   inaccessible directory.  Hence the second diagnostic, below.
+		   However it is also possible that F is a non-directory.
+		   That can happen when we use the `! ROOT_CAN_UNLINK_DIRS'
+		   block of code and when DO_UNLINK fails due to EPERM.
+		   In that case, give a better diagnostic.  */
+		if (errno == ENOTDIR)
+		  error (0, saved_errno, _("cannot remove %s"),
+			 quote (full_filename (f)));
+		else
+		  error (0, errno, _("cannot chdir from %s to %s"),
+			 quote_n (0, full_filename (".")), quote_n (1, f));
+		AD_mark_as_unremovable (ds, f);
+		status = RM_ERROR;
+		break;
+	      }
+	    if (cycle_check (&ds->cycle_check_state, subdir_sb))
+	      {
+		error (0, 0, _("\
 WARNING: Circular directory structure.\n\
 This almost certainly means that you have a corrupted file system.\n\
 NOTIFY YOUR SYSTEM MANAGER.\n\
 The following directory is part of the cycle:\n  %s\n"),
-		     quote (full_filename (".")));
-	      longjmp (ds->current_arg_jumpbuf, 1);
-	    }
+		       quote (full_filename (".")));
+		longjmp (ds->current_arg_jumpbuf, 1);
+	      }
 
-	  *subdir = xstrdup (f);
-	  break;
+	    *subdir = xstrdup (f);
+	    break;
+	  }
 	}
 
       /* Record status for this directory.  */
@@ -948,7 +976,10 @@ remove_dir (Dirstack_state *ds, char const *dir, struct saved_cwd **cwd_state,
     {
       *cwd_state = XMALLOC (struct saved_cwd, 1);
       if (save_cwd (*cwd_state))
-	return RM_ERROR;
+	{
+	  error (0, errno, _("cannot get current directory"));
+	  return RM_ERROR;
+	}
       AD_push_initial (ds, *cwd_state);
       AD_INIT_OTHER_MEMBERS ();
     }
@@ -1093,10 +1124,13 @@ enum RM_status
 rm (size_t n_files, char const *const *file, struct rm_options const *x)
 {
   struct saved_cwd *cwd_state = NULL;
-  enum RM_status status = RM_OK;
-  size_t i;
   Dirstack_state *ds;
 
+  /* Put the following two variables in static storage, so they can't
+     be clobbered by the potential longjmp into this function.  */
+  static enum RM_status status = RM_OK;
+  static size_t i;
+
   ds = ds_init ();
 
   for (i = 0; i < n_files; i++)
